@@ -1,9 +1,10 @@
 use super::*;
+use crate::assert_parser;
 
 #[test]
 fn test_date_year() {
     assert_eq!(Ok((&[][..], 2015)), date_year(b"2015"));
-    assert_eq!(Ok((&[][..], -0333)), date_year(b"-0333"));
+    assert_eq!(Ok((&[][..], -333)), date_year(b"-0333"));
     assert_eq!(Ok((&b"-"[..], 2015)), date_year(b"2015-"));
     assert!(date_year(b"abcd").is_err());
     assert!(date_year(b"2a03").is_err());
@@ -276,6 +277,59 @@ fn test_duration_datetime_error() {
     assert!(duration_datetime(b"0001-02-03T04:05:06").is_err()); // missing P at start
 }
 
+#[rustfmt::skip]
+#[test]
+fn duration_roundtrip() {
+    assert_parser!(
+        parse_duration, "P0W", Duration::Weeks(0)
+    );
+
+    assert_parser!(
+        parse_duration, "P2021Y11M16DT23H26M59.123S",
+        Duration::YMDHMS { year: 2021, month: 11, day: 16, hour: 23, minute: 26, second: 59, millisecond: 123 }
+    );
+    assert_parser!(
+        parse_duration, "P2021Y11M16DT23H26M59S",
+        Duration::YMDHMS { year: 2021, month: 11, day: 16, hour: 23, minute: 26, second: 59, millisecond: 0 }
+    );
+    assert_parser!(
+        parse_duration, "P2021Y11M16DT23H26M",
+        Duration::YMDHMS { year: 2021, month: 11, day: 16, hour: 23, minute: 26, second: 0, millisecond: 0 }
+    );
+    assert_parser!(
+        parse_duration, "P2021Y11M16DT23H",
+        Duration::YMDHMS { year: 2021, month: 11, day: 16, hour: 23, minute: 0, second: 0, millisecond: 0 }
+    );
+    assert_parser!(
+        parse_duration, "P2021Y11M16D",
+        Duration::YMDHMS { year: 2021, month: 11, day: 16, hour: 0, minute: 0, second: 0, millisecond: 0 }
+    );
+    assert_parser!(
+        parse_duration, "P2021Y11M16DT1S",
+        Duration::YMDHMS { year: 2021, month: 11, day: 16, hour: 0, minute: 0, second: 1, millisecond: 0 }
+    );
+    assert_parser!(
+        parse_duration, "P2021Y11M16DT0.471S",
+        Duration::YMDHMS { year: 2021, month: 11, day: 16, hour: 0, minute: 0, second: 0, millisecond: 471 }
+    );
+    assert_parser!(
+        parse_duration, "P2021Y11M",
+        Duration::YMDHMS { year: 2021, month: 11, day: 0, hour: 0, minute: 0, second: 0, millisecond: 0 }
+    );
+    assert_parser!(
+        parse_duration, "P11M",
+        Duration::YMDHMS { year: 0, month: 11, day: 0, hour: 0, minute: 0, second: 0, millisecond: 0 }
+    );
+    assert_parser!(
+        parse_duration, "P16D",
+        Duration::YMDHMS { year: 0, month: 0, day: 16, hour: 0, minute: 0, second: 0, millisecond: 0 }
+    );
+    assert_parser!(
+        parse_duration, "P0D",
+        Duration::YMDHMS { year: 0, month: 0, day: 0, hour: 0, minute: 0, second: 0, millisecond: 0 }
+    );
+}
+
 // #[test]
 // fn corner_cases() {
 //    // how to deal with left overs?
