@@ -1,6 +1,3 @@
-extern crate iso8601;
-extern crate nom;
-
 use iso8601::*;
 
 #[test]
@@ -106,9 +103,9 @@ fn test_millisecond() {
 
     assert_eq!(
         Ok(Time {
-            hour: 04,
-            minute: 05,
-            second: 06,
+            hour:4,
+            minute:5,
+            second:6,
             millisecond: 123,
             tz_offset_hours: 0,
             tz_offset_minutes: 0
@@ -120,13 +117,13 @@ fn test_millisecond() {
         Ok(DateTime {
             date: Date::Week {
                 year: 2001,
-                ww: 05,
+                ww:5,
                 d: 6
             },
             time: Time {
-                hour: 04,
-                minute: 05,
-                second: 06,
+                hour:4,
+                minute:5,
+                second:6,
                 millisecond: 123,
                 tz_offset_hours: 0,
                 tz_offset_minutes: 0
@@ -368,7 +365,7 @@ fn short_dtim1() {
         Ok(DateTime {
             date: Date::YMD {
                 year: 2007,
-                month: 08,
+                month:8,
                 day: 31,
             },
             time: Time {
@@ -389,7 +386,7 @@ fn short_dtim2() {
         Ok(DateTime {
             date: Date::YMD {
                 year: 2007,
-                month: 08,
+                month:8,
                 day: 31,
             },
             time: Time {
@@ -549,7 +546,6 @@ fn test_ordinal_date() {
         date("2015-057")
     );
 
-    // not valid, but this should be tested elsewhere
     assert_eq!(
         Ok(Date::Ordinal {
             year: 2015,
@@ -560,17 +556,11 @@ fn test_ordinal_date() {
     assert_eq!(
         Ok(Date::Ordinal {
             year: 2015,
-            ddd: 399,
+            ddd: 366,
         }),
-        date("2015-399")
-    );
-    assert_eq!(
-        Ok(Date::Ordinal {
-            year: 2015,
-            ddd: 000,
-        }),
-        date("2015-000")
+        date("2015-366")
     );
+    assert_eq!(Ok(Date::Ordinal { year: 2015, ddd: 1 }), date("2015-001"));
 
     // not valid here either
     assert!(date("2015-400").is_err());
@@ -615,7 +605,7 @@ fn test_datetime_correct() {
         Ok(DateTime {
             date: Date::YMD {
                 year: 2006,
-                month: 08,
+                month:8,
                 day: 31
             },
             time: Time {
@@ -633,7 +623,7 @@ fn test_datetime_correct() {
         Ok(DateTime {
             date: Date::YMD {
                 year: 2007,
-                month: 08,
+                month:8,
                 day: 31
             },
             time: Time {
@@ -651,7 +641,7 @@ fn test_datetime_correct() {
         Ok(DateTime {
             date: Date::YMD {
                 year: 2007,
-                month: 08,
+                month:8,
                 day: 31
             },
             time: Time {
@@ -669,7 +659,7 @@ fn test_datetime_correct() {
         Ok(DateTime {
             date: Date::YMD {
                 year: 2007,
-                month: 08,
+                month:8,
                 day: 31
             },
             time: Time {
@@ -687,8 +677,8 @@ fn test_datetime_correct() {
         Ok(DateTime {
             date: Date::YMD {
                 year: 2009,
-                month: 02,
-                day: 01
+                month: 2,
+                day: 1
             },
             time: Time {
                 hour: 9,
@@ -723,7 +713,7 @@ fn test_datetime_correct() {
         Ok(DateTime {
             date: Date::YMD {
                 year: 2011,
-                month: 06,
+                month:6,
                 day: 30
             },
             time: Time {
@@ -741,12 +731,12 @@ fn test_datetime_correct() {
         Ok(DateTime {
             date: Date::YMD {
                 year: 2015,
-                month: 06,
+                month:6,
                 day: 29
             },
             time: Time {
                 hour: 23,
-                minute: 07,
+                minute:7,
                 second: 0,
                 millisecond: 0,
                 tz_offset_hours: 2,
@@ -759,7 +749,7 @@ fn test_datetime_correct() {
         Ok(DateTime {
             date: Date::YMD {
                 year: 2015,
-                month: 06,
+                month:6,
                 day: 26
             },
             time: Time {
@@ -777,7 +767,7 @@ fn test_datetime_correct() {
         Ok(DateTime {
             date: Date::YMD {
                 year: 2015,
-                month: 06,
+                month:6,
                 day: 26
             },
             time: Time {
@@ -795,11 +785,11 @@ fn test_datetime_correct() {
         Ok(DateTime {
             date: Date::Week {
                 year: 2015,
-                ww: 05,
+                ww:5,
                 d: 6
             },
             time: Time {
-                hour: 04,
+                hour:4,
                 minute: 5,
                 second: 6,
                 millisecond: 0,
@@ -813,11 +803,11 @@ fn test_datetime_correct() {
         Ok(DateTime {
             date: Date::Week {
                 year: 2015,
-                ww: 05,
+                ww:5,
                 d: 6
             },
             time: Time {
-                hour: 04,
+                hour:4,
                 minute: 5,
                 second: 6,
                 millisecond: 0,
@@ -834,7 +824,7 @@ fn test_datetime_correct() {
                 ddd: 56
             },
             time: Time {
-                hour: 04,
+                hour:4,
                 minute: 5,
                 second: 6,
                 millisecond: 0,
@@ -851,7 +841,7 @@ fn test_datetime_correct() {
                 ddd: 56
             },
             time: Time {
-                hour: 04,
+                hour:4,
                 minute: 5,
                 second: 6,
                 millisecond: 0,
@@ -900,13 +890,13 @@ fn test_datetime_correct() {
         Ok(DateTime {
             date: Date::Week {
                 year: 2001,
-                ww: 05,
+                ww:5,
                 d: 6
             },
             time: Time {
-                hour: 04,
-                minute: 05,
-                second: 06,
+                hour:4,
+                minute:5,
+                second:6,
                 millisecond: 123,
                 tz_offset_hours: 0,
                 tz_offset_minutes: 0
@@ -918,13 +908,13 @@ fn test_datetime_correct() {
         Ok(DateTime {
             date: Date::Week {
                 year: 2001,
-                ww: 05,
+                ww:5,
                 d: 6
             },
             time: Time {
-                hour: 04,
-                minute: 05,
-                second: 06,
+                hour:4,
+                minute:5,
+                second:6,
                 millisecond: 123,
                 tz_offset_hours: 0,
                 tz_offset_minutes: 0
@@ -966,3 +956,234 @@ fn issue12_regression_2() {
         time(input)
     );
 }
+
+#[test]
+fn test_duration_ymdhms() {
+    use core::time::Duration as StdDuration;
+
+    // full YMDHMS
+    let dur = duration("P1Y2M3DT4H5M6S").unwrap();
+    assert_eq!(
+        Duration::YMDHMS {
+            year: 1,
+            month: 2,
+            day: 3,
+            hour: 4,
+            minute: 5,
+            second: 6,
+            millisecond: 0,
+        },
+        dur
+    );
+    assert_eq!(StdDuration::from(dur), StdDuration::new(36993906, 0));
+
+    // full YMDHMS with milliseconds dot delimiter
+    let dur = duration("P1Y2M3DT4H5M6.7S").unwrap();
+    assert_eq!(
+        Duration::YMDHMS {
+            year: 1,
+            month: 2,
+            day: 3,
+            hour: 4,
+            minute: 5,
+            second: 6,
+            millisecond: 700,
+        },
+        dur
+    );
+    assert_eq!(
+        StdDuration::from(dur),
+        StdDuration::new(36993906, 700000000)
+    );
+
+    // full YMDHMS with milliseconds comma delimiter
+    let dur = duration("P1Y2M3DT4H5M6,7S").unwrap();
+    assert_eq!(
+        Duration::YMDHMS {
+            year: 1,
+            month: 2,
+            day: 3,
+            hour: 4,
+            minute: 5,
+            second: 6,
+            millisecond: 700,
+        },
+        dur
+    );
+    assert_eq!(
+        StdDuration::from(dur),
+        StdDuration::new(36993906, 700000000)
+    );
+
+    // subset YM-HM-
+    let dur = duration("P1Y2MT4H5M").unwrap();
+    assert_eq!(
+        Duration::YMDHMS {
+            year: 1,
+            month: 2,
+            day: 0,
+            hour: 4,
+            minute: 5,
+            second: 0,
+            millisecond: 0,
+        },
+        dur
+    );
+    assert_eq!(StdDuration::from(dur), StdDuration::new(36734700, 0));
+
+    // subset Y-----
+    let dur = duration("P1Y").unwrap();
+    assert_eq!(
+        Duration::YMDHMS {
+            year: 1,
+            month: 0,
+            day: 0,
+            hour: 0,
+            minute: 0,
+            second: 0,
+            millisecond: 0,
+        },
+        dur
+    );
+    assert_eq!(StdDuration::from(dur), StdDuration::new(31536000, 0));
+
+    // subset ---H--
+    let dur = duration("PT4H").unwrap();
+    assert_eq!(
+        Duration::YMDHMS {
+            year: 0,
+            month: 0,
+            day: 0,
+            hour: 4,
+            minute: 0,
+            second: 0,
+            millisecond: 0,
+        },
+        dur
+    );
+    assert_eq!(StdDuration::from(dur), StdDuration::new(14400, 0));
+
+    // subset -----S with milliseconds dot delimiter
+    let dur = duration("PT6.7S").unwrap();
+    assert_eq!(
+        Duration::YMDHMS {
+            year: 0,
+            month: 0,
+            day: 0,
+            hour: 0,
+            minute: 0,
+            second: 6,
+            millisecond: 700,
+        },
+        dur
+    );
+    assert_eq!(StdDuration::from(dur), StdDuration::new(6, 700000000));
+
+    // subset -----S with milliseconds comma delimiter
+    let dur = duration("PT6,700S").unwrap();
+    assert_eq!(
+        Duration::YMDHMS {
+            year: 0,
+            month: 0,
+            day: 0,
+            hour: 0,
+            minute: 0,
+            second: 6,
+            millisecond: 700,
+        },
+        dur
+    );
+    assert_eq!(StdDuration::from(dur), StdDuration::new(6, 700000000));
+
+    // empty duration, using Y
+    let dur = duration("P0Y").unwrap();
+    assert_eq!(
+        Duration::YMDHMS {
+            year: 0,
+            month: 0,
+            day: 0,
+            hour: 0,
+            minute: 0,
+            second: 0,
+            millisecond: 0,
+        },
+        dur
+    );
+    assert_eq!(StdDuration::from(dur), StdDuration::new(0, 0));
+
+    // empty duration, using S
+    let dur = duration("PT0S").unwrap();
+    assert_eq!(
+        Duration::YMDHMS {
+            year: 0,
+            month: 0,
+            day: 0,
+            hour: 0,
+            minute: 0,
+            second: 0,
+            millisecond: 0,
+        },
+        dur
+    );
+    assert_eq!(StdDuration::from(dur), StdDuration::new(0, 0));
+
+    let dur = duration("PT42M30S").unwrap();
+    assert_eq!(
+        Duration::YMDHMS {
+            year: 0,
+            month: 0,
+            day: 0,
+            hour: 0,
+            minute: 42,
+            second: 30,
+            millisecond: 0,
+        },
+        dur
+    );
+    assert_eq!(StdDuration::from(dur), StdDuration::new(2550, 0));
+
+    let dur = duration("P0001-02-03T04:05:06").unwrap();
+    assert_eq!(
+        Duration::YMDHMS {
+            year: 1,
+            month: 2,
+            day: 3,
+            hour: 4,
+            minute: 5,
+            second: 6,
+            millisecond: 0,
+        },
+        dur
+    );
+    assert_eq!(StdDuration::from(dur), StdDuration::new(36993906, 0));
+
+    let dur = duration("P2018-04-27T00:00:00").unwrap();
+    assert_eq!(
+        Duration::YMDHMS {
+            year: 2018,
+            month: 4,
+            day: 27,
+            hour: 0,
+            minute: 0,
+            second: 0,
+            millisecond: 0,
+        },
+        dur
+    );
+    assert_eq!(StdDuration::from(dur), StdDuration::new(63652348800, 0));
+}
+
+#[test]
+fn test_duration_weeks() {
+    use core::time::Duration as StdDuration;
+
+    let dur = duration("P0W").unwrap();
+    assert_eq!(Duration::Weeks(0), dur);
+    assert_eq!(StdDuration::from(dur), StdDuration::new(0, 0));
+    let dur = duration("P26W").unwrap();
+    assert_eq!(Duration::Weeks(26), dur);
+    assert_eq!(StdDuration::from(dur), StdDuration::new(15724800, 0));
+    let dur = duration("P52W").unwrap();
+    assert_eq!(Duration::Weeks(52), dur);
+    assert_eq!(StdDuration::from(dur), StdDuration::new(31449600, 0));
+}
